% computes bounds for psi as in Benzi-Golub
% (actually these are lower bounds on the bounds,
% because we are estimating M by taking the abs value
% of psi along the imaginary axis...)

% recall that psi has poles at 2*pi*i*n,
% so the Bernstein ellipse cannot extend past +- 2*pi*i
% this means beta<2pi

% chi is a real number, 1<chi<2pi+sqrt(4pi^2+1), best results with chi=12
% k  vector of indices, e.g. 1:100,   (differences |i-j|)
% b vector of bounds for the corresponding indices

function b=bg_bound(chi,k)

beta=(chi^2-1)/(2*chi);
M=abs(beta/(exp(1i*beta)-1));
b=2*chi*M./((chi-1)*(chi.^k));
end