% Compute bounds on decay of psi_1(A)
% using the mixed polynomial-rational approximation;
% compare with bounds from Benzi-Golub 
% and with actual behavior of matrix function.
%
% Here A is a tridiagonal Toeplitz matrix with spectrum in [-1,1].

% Choose n and s (notation as in the manuscript).
n=2;
s=200;

% Define A and m (bandwidth).
d=100;
A=full(gallery('tridiag',d,-1,3,-1));
A=A/norm(A);
m=1;

eigA=eig(A);
maxeig=max(eigA);
mineig=min(eigA);

% Compute bounds from manuscript.
ourbound=zeros(1,d);

Ascalepower=(A/(2*pi))^(2*n+2);

% Compute bound on r_{n,s}, before product with power of scaled A

ratbndmatrix=zeros(d);
for i=1:d
    for j=i:d
        sumk=0;
        for k=1:s
          ak=(mineig/(2*pi))^2+k^2;  % slight improvement to (3.35)
          bk=(maxeig/(2*pi))^2+k^2;
          srk=sqrt(bk/ak);
          lambdak=((srk-1)/(srk+1))^(1/(2*m));
          Ck=max(1/ak, (1+srk)^2/(2*ak*srk^2));
          sumk=sumk+Ck*lambdak^(abs(i-j))/(k^(2*n));
        end
       ratbndmatrix(i,j)=sumk;
       ratbndmatrix(j,i)=sumk;
    end
end

% Multiply with power of A:

ratbndmatrix2=2*abs(Ascalepower)*ratbndmatrix;

% More conservatively: multiply with power of norm(A):

Acoeff=norm(A/(2*pi))^(2*n+2);
rr=[ones(1,2*m*(n+1)+1),zeros(1,d-2*m*(n+1)-1)];
Acoeffmatrix=Acoeff*toeplitz(rr',rr);
ratbndmatrix1=2*Acoeffmatrix*ratbndmatrix;


% Compute bound on epsilon_{n,s}

sumzeta=0;
for k=1:s
    sumzeta=sumzeta+1/(k^(2*n+2));
end

epsbound=2*Acoeff*(zeta(2*n+2)-sumzeta);

% Sum the two terms

ourboundmatrix1=ratbndmatrix1+epsbound;
ourboundmatrix2=ratbndmatrix2+epsbound;

% Plot results for the first relevant ell+1 indices.
ell=20;
figure(1)
plot(log10(ourboundmatrix1(1,2*n*m+2:2*n*m+2+ell)),'m*-')
hold on
plot(log10(ourboundmatrix2(1,2*n*m+2:2*n*m+2+ell)),'rd-')

% Compute actual behavior of matrix function and plot
B=A/(expm(A)-eye(d));
plot(log10(abs(B(1,2*n*m+2:2*n*m+2+ell))),'ko-')

% Compute Benzi-Golub bounds and plot

BG=bg_bound(12,2*n*m+2:d);
plot(log10(abs(BG(1:ell+1))),'gx-')
