% Multiplication of two nxn quasiseparable matrices
% (block size 1)
% Implements Thm 17.6 in Yuli's book (vol.1).
%
function [p,q,a,g,h,b,d] = qs_product(p1,q1,a1,g1,h1,b1,d1,p2,q2,a2,g2,h2,b2,d2)

n=length(d1);

[~,rL] = size(p1); % lower qs order of first matrix
[rU,~] = size(h1); % upper qs order of first matrix
[~,sL] = size(p2); % lower qs order of second matrix
[sU,~] = size(h2); % upper qs order of second matrix
tL = rL + sL;
tU = rU + sU;
p = zeros(n,tL);
q = zeros(tL,n);
a = zeros(tL,tL,n);
g = zeros(n,tU);
h = zeros(tU,n);
b = zeros(tU,tU,n);
d = zeros(n,1);

beta = zeros(rL,sU);
gamma = zeros(rU,sL);
for k = 1:n
    M = [p1(k,:) 0; a1(:,:,k) q1(:,k)] * [beta zeros(rL,1); zeros(1,sU) 1] *...
       [h2(:,k) b2(:,:,k); 0 g2(k,:)];
   dp(k) = M(1,1);
   gt(k,:) = M(1,2:end);
   qt(:,k) = M(2:end,1);
   beta = M(2:end,2:end);
end
for k = n:-1:1
    N = [h1(:,k) b1(:,:,k); d1(k) g1(k,:)] * [1 zeros(1,sL); zeros(rU,1) gamma] *...
        [p2(k,:) d2(k); a2(:,:,k) q2(:,k)];
    gamma = N(1:rU,1:sL);
    ht(:,k) = N(1:rU,sL+1);
    pt(k,:) = N(rU+1,1:sL);
    dpp(k) = N(rU+1,sL+1);
end

for k = 1:n
    p(k,:) = [p1(k,:), pt(k,:)];
    q(:,k) = [q1(:,k)*d2(k) + qt(:,k); q2(:,k)];
    a(:,:,k) = [a1(:,:,k) q1(:,k)*p2(k,:); zeros(rL,sL) a2(:,:,k)];
    g(k,:) = [d1(k)*g2(k,:) + gt(k,:) g1(k,:)];
    h(:,k) = [h2(:,k); ht(:,k)];
    b(:,:,k) = [b2(:,:,k) zeros(sU,rU); h1(:,k)*g2(k,:) b1(:,:,k)];
    d(k) = dp(k) + dpp(k);
end